unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, DB, VKDBFDataSet, VKDBFIndex, VKDBFNTX, StdCtrls;

type
  TForm1 = class(TForm)
    CreateTable: TButton;
    OpenTableAndCreateIndex: TButton;
    FillTableWithData: TButton;
    SeekAndFindKey: TButton;
    UpdateTable: TButton;
    LocateAndLookup: TButton;
    procedure CreateTableClick(Sender: TObject);
    procedure OpenTableAndCreateIndexClick(Sender: TObject);
    procedure FillTableWithDataClick(Sender: TObject);
    procedure UpdateTableClick(Sender: TObject);
    procedure SeekAndFindKeyClick(Sender: TObject);
    procedure LocateAndLookupClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}


//Create Table
procedure TForm1.CreateTableClick(Sender: TObject);
var
  dbftab: TVKDBFNTX;
begin

  dbftab := TVKDBFNTX.Create(self);

  try

    dbftab.DBFFileName := 'sample.dbf';

    // Fill fields
    with dbftab.DBFFieldDefs.Add as TVKDBFFieldDef do begin
      Name := 'F1';
      field_type := 'N'; //Standard Dbase III type
      len := 10;
    end;
    with dbftab.DBFFieldDefs.Add as TVKDBFFieldDef do begin
      Name := 'F2';
      field_type := 'C'; //Standard Dbase III type
      len := 20;
    end;
    with dbftab.DBFFieldDefs.Add as TVKDBFFieldDef do begin
      Name := 'F3';
      field_type := 'N'; //Standard Dbase III type
      len := 10;
      dec := 3;
    end;
    with dbftab.DBFFieldDefs.Add as TVKDBFFieldDef do begin
      Name := 'F4';
      field_type := 'M'; //Standard Dbase III type
    end;
    with dbftab.DBFFieldDefs.Add as TVKDBFFieldDef do begin
      Name := 'F5';
      field_type := 'E'; //Extended VKDBF type !!!
      extend_type := dbftR8;   //Real 8 bytes type without NULL
    end;
    with dbftab.DBFFieldDefs.Add as TVKDBFFieldDef do begin
      Name := 'F6';
      field_type := 'E'; //Extended VKDBF type !!!
      extend_type := dbftBlob; //Binary lage object
    end;

    // Define indexes

    // First define index bag
    with dbftab.DBFIndexDefs.Add as TVKNTXBag do begin
      IndexFileName := 'sample.ntx';
      // In bag you can define orders, but in NTX you can define onle one order !
      with Orders.Add as TVKNTXOrder do begin
        Name := 'ORDER1';
        KeyExpresion := 'STR(F1) + F2';
      end;
    end;

    // Now we are create table sample.dbf with sample.dbt lob storage and
    // sample.ntx index bag with one order 'ORDER1'
    dbftab.CreateTable;

    // or that
    //dbftab.CreateNow := true;
    //dbftab.CreateNow := false;

    ShowMessage('Ok');

  finally

    FreeAndNil(dbftab);

  end;

end;

//Open table
procedure TForm1.OpenTableAndCreateIndexClick(Sender: TObject);
var
  dbftab: TVKDBFNTX;
begin

  dbftab := TVKDBFNTX.Create(self);

  try

    dbftab.DBFFileName := 'sample.dbf';
    dbftab.AccessMode.AccessMode := 66;    //ReadWrite and Share
    with dbftab.DBFIndexDefs.Add as TVKNTXBag do IndexFileName := 'sample.ntx';
    dbftab.Open;
    //or
    //dbftab.Active := true;

    // Make 'ORDER1' active
    dbftab.Orders[0].Active := True;
    //or
    //dbftab.SetOrder('ORDER1');
    //or
    //dbftab.IndexName := 'ORDER1';

    // Make table in phisical record order
    dbftab.SetOrder(0);

    //Create new order
    with dbftab.Indexes.Add as TVKNTXIndex do begin
      NTXFileName := 'sample1.ntx';
      KeyExpresion := 'F3';
      CreateIndex(True);
      //or
      //CreateCompactIndex();
    end;

    ShowMessage('Ok');

  finally

    //Close table
    dbftab.Close;
    //or
    //dbftab.Active := false;

    // but close table is not necessary because
    // dbftab.Free automaticaly close one
    FreeAndNil(dbftab);

  end;

end;

//Fill table with data
procedure TForm1.FillTableWithDataClick(Sender: TObject);
var
  dbftab: TVKDBFNTX;
  i: Integer;
begin

  dbftab := TVKDBFNTX.Create(self);

  try

    dbftab.DBFFileName := 'sample.dbf';
    dbftab.AccessMode.AccessMode := 66;    //ReadWrite and Share
    with dbftab.DBFIndexDefs.Add as TVKNTXBag do IndexFileName := 'sample.ntx';
    dbftab.Active := true;

    // if we use Share mode we have to lock table
    dbftab.FLock;  //lock table from ather net users
    //switch on BUFFER
    // !! use add buffer much more faster !!
    dbftab.BeginAddBuffered(100);
    try
      for i := 1 to 1000 do begin
        dbftab.Append;
        dbftab.FieldByName('F1').AsInteger := 1001 - i;
        dbftab.Fields[1].AsString := IntToStr(i);  //F2
        dbftab['F3'] := 2001 - i * 2;
        dbftab['F4'] := 'o-ops: ' + IntToStr(i);
        dbftab['F5'] := (2001 - i * 2)/1000;
        dbftab.Post;
        FillTableWithData.Caption := IntToStr(i);
      end;
    finally
      dbftab.EndAddBuffered;  // close add buffer
      dbftab.UnLock;          // unlock table for ather users
    end;

  finally

    FreeAndNil(dbftab);

  end;

end;

// Update Table
procedure TForm1.UpdateTableClick(Sender: TObject);
var
  dbftab: TVKDBFNTX;
begin

  dbftab := TVKDBFNTX.Create(self);

  try

    dbftab.DBFFileName := 'sample.dbf';
    dbftab.AccessMode.AccessMode := 66;    //ReadWrite and Share
    with dbftab.DBFIndexDefs.Add as TVKNTXBag do IndexFileName := 'sample.ntx';
    dbftab.Active := true;

    // Make table in phisical record order
    dbftab.SetOrder(0);

    // if we use Share mode we have to lock table
    dbftab.FLock;  //lock table from ather net users
    try
      dbftab.First;
      while not dbftab.Eof do begin
        dbftab.Edit;
        dbftab.Fields[1].AsString := IntToStr(3001 - dbftab.RecNo);  //F2
        dbftab.Post;
        UpdateTable.Caption := IntToStr(dbftab.RecNo);
        dbftab.Next;
      end;
    finally
      dbftab.UnLock;          // unlock table for ather users
    end;

  finally

    FreeAndNil(dbftab);

  end;

end;

//Seek and find key
procedure TForm1.SeekAndFindKeyClick(Sender: TObject);
var
  dbftab: TVKDBFNTX;
  r: integer;

  procedure ShowCurrent;
  begin
    if r > 0 then begin
      dbftab.RecNo := r;
      ShowMessage('Rec = ' + IntToStr(r) + #13#10 +
                  'F1 = ' + dbftab.Fields[0].AsString + #13#10 +
                  'F2 = ' + dbftab.Fields[1].AsString + #13#10 +
                  'F4 = ' + dbftab.Fields[3].AsString);
    end else
      ShowMessage('Key not found');
    dbftab.RecNo := 1;
  end;

begin

  dbftab := TVKDBFNTX.Create(self);

  try

    dbftab.DBFFileName := 'sample.dbf';
    dbftab.AccessMode.AccessMode := 66;    //ReadWrite and Share
    with dbftab.DBFIndexDefs.Add as TVKNTXBag do IndexFileName := 'sample.ntx';
    dbftab.Active := true;

    // Make table in phisical record order
    dbftab.SetOrder(0);

    // For find string key you can use some of the following code.
    // You should notes that some method positioned record into DataSet
    // some method not.

    // 1)
    r := dbftab.Orders[0].FindKey('       5982598', true);
    ShowCurrent;

    // 2)
    r := dbftab.Orders[0].FindKeyFields('F1;F2', [598, 2598]);
    ShowCurrent;

    // 3)
    if dbftab.Orders[0].Seek('       5982598', true) then
      r := dbftab.RecNo
    else
      r := 0;
    ShowCurrent;

    // 4)
    if dbftab.Orders[0].SeekFirst('       5982598', false, true) then
      r := dbftab.RecNo
    else
      r := 0;
    ShowCurrent;

    // 5)
    r := dbftab.Orders[0].SeekFirstRecord('       5982598', false, true);
    ShowCurrent;

    // 6)
    r := dbftab.Orders[0].SeekFields('F1;F2', VarArrayOf([598, 2598]), false, true);
    ShowCurrent;

  finally

    FreeAndNil(dbftab);

  end;

end;

//Locate and lookup
procedure TForm1.LocateAndLookupClick(Sender: TObject);
var
  dbftab: TVKDBFNTX;
  f1: integer;

  procedure ShowCurrent;
  begin
    ShowMessage('Rec = ' + IntToStr(dbftab.RecNo) + #13#10 +
                'F1 = ' + dbftab.Fields[0].AsString + #13#10 +
                'F2 = ' + dbftab.Fields[1].AsString + #13#10 +
                'F4 = ' + dbftab.Fields[3].AsString);
  end;

begin

  dbftab := TVKDBFNTX.Create(self);

  try

    dbftab.DBFFileName := 'sample.dbf';
    dbftab.AccessMode.AccessMode := 66;    //ReadWrite and Share
    with dbftab.DBFIndexDefs.Add as TVKNTXBag do IndexFileName := 'sample.ntx';
    dbftab.Active := true;

    // Make table in phisical record order
    dbftab.SetOrder(0);

    // You can use some of the following code.

    // This locate will use order 'ORDER1' with key 'STR(F1) + F2'
    dbftab.Locate('F1;F2', VarArrayOf([598, 2598]), [loPartialKey]);
    ShowCurrent;

    // This locate will use order 'ORDER1' with key 'STR(F1) + F2'
    dbftab.Locate('F1', VarArrayOf([598]), [loPartialKey]);
    ShowCurrent;

    // This locate will use full scan table
    dbftab.Locate('F2;F1', VarArrayOf([2598, 598]), [loPartialKey]);
    ShowCurrent;

    // Lookup the same as Locate, but not positioned DataSet:
    dbftab.LookupOptions := [loPartialKey];
    f1 := dbftab.Lookup('F1;F2', VarArrayOf([598, 2598]), 'F1');
    ShowMessage(IntToStr(f1));

  finally

    FreeAndNil(dbftab);

  end;

end;

end.
